"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tick = exports.Ticks = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
function Ticks(props) {
    const { hidden, ticks, scale, getPositions, scientific = true, hiddenSecondaryTicks = false, ...otherProps } = props;
    if (hidden)
        return null;
    // Primary Ticks
    let elements = ticks.map((tick) => {
        const { line, text } = getPositions(scale(tick));
        return (jsx_runtime_1.jsx(Tick, Object.assign({ line: line, text: text }, otherProps, { children: scientific ? tick.toExponential(2) : tick }), tick));
    });
    if (!hiddenSecondaryTicks) {
        // generate secondaryTicks according to the density of primaryTicks
        const range = Math.abs((scale === null || scale === void 0 ? void 0 : scale.range()[1]) - (scale === null || scale === void 0 ? void 0 : scale.range()[0])) || 0;
        const mainTicksDensity = range / ticks.length;
        const density = mainTicksDensity < 50 ? 5 : 10;
        let secTicks = (scale === null || scale === void 0 ? void 0 : scale.ticks(ticks.length * density)) || [];
        // calculate middle tick position ([0,1]=> middle tick == 0.5)
        let middleTick = 0;
        let secTicksPerInterval = 0;
        if (ticks.length > 1) {
            let i = 0;
            while (secTicks[i] < ticks[1]) {
                while (secTicks[i] < ticks[0]) {
                    i++;
                    middleTick++;
                }
                secTicksPerInterval++;
                i++;
            }
        }
        const middleTickEnabled = secTicksPerInterval % 2 === 0 && secTicksPerInterval > 0;
        middleTick = middleTick - secTicksPerInterval / 2 + 1;
        // add secondaryTicks to the elements array
        const secElements = secTicks.map((tick) => {
            middleTick--;
            let strokeHeight = middleTickEnabled && middleTick === 0 ? 1 : 0.6;
            // exclude the main ticks
            if (ticks.includes(tick)) {
                middleTick = secTicksPerInterval / 2;
                return null;
            }
            const { line, text } = getPositions(scale(tick));
            return (jsx_runtime_1.jsx(Tick, Object.assign({ line: line, text: text, secondary: true, strokeHeight: strokeHeight }, otherProps), tick));
        }) || [];
        elements = [...secElements, ...elements];
    }
    return jsx_runtime_1.jsx(jsx_runtime_1.Fragment, { children: elements }, void 0);
}
exports.Ticks = Ticks;
function Tick(props) {
    const { line: { x1: lineX1 = 0, x2: lineX2 = 0, y1: lineY1 = 0, y2: lineY2 = 0 }, text: { x1: textX1 = 0, y1: textY1 = 0 }, style, children, strokeColor = 'black', strokeHeight = 1, anchor = 'end', secondary = false, } = props;
    return (jsx_runtime_1.jsxs("g", { children: [jsx_runtime_1.jsx("line", { x1: lineX1, x2: secondary && lineX1 !== lineX2 ? lineX2 * strokeHeight : lineX2, y1: lineY1, y2: secondary && lineY1 !== lineY2 ? lineY2 * strokeHeight : lineY2, stroke: strokeColor, strokeWidth: secondary ? 1 : 1.5 }, void 0),
            !secondary && (jsx_runtime_1.jsx("text", Object.assign({ x: textX1, y: textY1, textAnchor: anchor, dominantBaseline: "middle", style: { userSelect: 'none', ...style } }, { children: children }), void 0))] }, void 0));
}
exports.Tick = Tick;
