"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const hooks_1 = require("../hooks");
const utils_1 = require("../utils");
const ScatterSeries_1 = __importDefault(require("./ScatterSeries"));
const legendsContext_1 = require("./legendsContext");
function BarSeries(props) {
    var _a;
    const [, legendDispatch] = legendsContext_1.useLegend();
    const { colorScaler } = hooks_1.usePlotContext();
    const [id] = react_1.useState(() => props.groupId || `series-${utils_1.getNextId()}`);
    const { lineStyle = {}, displayMarker = false, ...otherProps } = props;
    const lineProps = {
        id,
        data: props.data,
        xAxis: props.xAxis || 'x',
        yAxis: props.yAxis || 'y',
        lineStyle,
    };
    const colorLine = (lineStyle === null || lineStyle === void 0 ? void 0 : lineStyle.stroke)
        ? lineStyle === null || lineStyle === void 0 ? void 0 : lineStyle.stroke.toString()
        : colorScaler(id);
    const color = ((_a = otherProps.markerStyle) === null || _a === void 0 ? void 0 : _a.fill.toString()) || colorScaler(id);
    const figure = otherProps.markerShape || 'circle';
    const shape = react_1.useMemo(() => {
        return {
            color,
            figure,
            hidden: !displayMarker,
        };
    }, [color, displayMarker, figure]);
    react_1.useEffect(() => {
        legendDispatch({
            type: 'ADD_LEGEND_LABEL',
            payload: {
                id,
                label: otherProps.label,
                colorLine,
                shape,
            },
        });
        return () => legendDispatch({ type: 'REMOVE_LEGEND_LABEL', payload: { id } });
    }, [colorLine, legendDispatch, otherProps.label, shape, id]);
    return (jsx_runtime_1.jsxs("g", { children: [props.hidden ? null : jsx_runtime_1.jsx(BarSeriesRender, Object.assign({}, lineProps), void 0),
            jsx_runtime_1.jsx(ScatterSeries_1.default, Object.assign({}, otherProps, { hidden: !displayMarker || props.hidden, groupId: id }), void 0)] }, void 0));
}
exports.default = BarSeries;
function BarSeriesRender({ id, data, xAxis, yAxis, lineStyle, }) {
    // Get scales from context
    const { axisContext, colorScaler } = hooks_1.usePlotContext();
    const [xScale, yScale] = utils_1.validateAxis(axisContext, xAxis, yAxis);
    const color = colorScaler(id);
    if ([xScale, yScale].includes(undefined))
        return null;
    // default style
    const style = {
        stroke: color,
        strokeWidth: 2,
        ...lineStyle,
    };
    return (jsx_runtime_1.jsx("g", { children: data.map(({ x, y }) => (jsx_runtime_1.jsx("line", { style: style, x1: xScale(x), x2: xScale(x), y1: yScale(y), y2: yScale(0) }, `${x}-${y}`))) }, void 0));
}
